#version 430 core

in vec3 texCoords;
uniform samplerCube cubemap; //cubemap texture

out vec4 color;

vec4 celShading(vec4 color);

void main() {
    color = vec4(texture(cubemap, texCoords).rgb, 1.0);
    color = celShading(color);
}

vec4 celShading(vec4 color){
    //Calculate RGB to HSV and change the V (Brightness value)
    float H, S, V;
    float Cmax = max(max(color.x, color.y), color.z);
    float Cmin = min(min(color.x, color.y), color.z);
    float delta = Cmax - Cmin;
    //Calculate V
    V = Cmax;
    //Calculate S
    if(Cmax == 0){
        S = 0;
    }else{
        S = delta / Cmax;
    }
    //Calculate H
    if(delta == 0){
        H = 0;
    }else if(Cmax == color.x){
        H = 0.6f * (mod(((color.y - color.z)/delta), 6.0f));
    }else if(Cmax == color.y){
        H = 0.6f * (((color.z - color.x)/delta)+2);
    }else if(Cmax == color.z){
        H = 0.6f * (((color.x - color.y)/delta)+4);
    }
    H = mod(H, 3.6f);
    H = H * 100;

    //Change V value
    float colorLevels = 8;

    float level = floor(V * colorLevels);
    V = level / colorLevels;

    //Now calculate back to RGB
    float R, G, B;

    float C = V * S;
    float X = C * (1 - abs(mod((H/60),2.0f) - 1));
    float m = V - C;

    if(H >= 0 && H < 60){
        R = C;
        G = X;
        B = 0;
    }else if(H >= 60 && H < 120){
        R = X;
        G = C;
        B = 0;
    }else if(H >= 120 && H < 180){
        R = 0;
        G = C;
        B = X;
    }else if(H >= 180 && H < 240){
        R = 0;
        G = X;
        B = C;
    }else if(H >= 240 && H < 300){
        R = X;
        G = 0;
        B = C;
    }else if(H >= 300 && H < 360){
        R = C;
        G = 0;
        B = X;
    }
    R = R + m;
    G = G + m;
    B = B + m;
    return vec4(R, G, B, color.a);
}