#version 430 core
layout (location=0) out vec4 FragColor;
  
in vec2 TexCoords;

layout(binding=0) uniform sampler2D colorTexture;
layout(binding=1) uniform sampler2D depthTexture;

//Kernels for edge detection
float sy[9] = float[](
	-1.0, -2.0, -1.0,
	0.0, 0.0, 0.0,
	1.0, 2.0, 1.0
	);


float sx[9] = float[](
	1.0, 0.0, -1.0,
	2.0, 0.0, -2.0,
	1.0, 0.0, -1.0
);

mat3 sobel_y = mat3( 
     1.0, 0.0, -1.0, 
     2.0, 0.0, -2.0, 
     1.0, 0.0, -1.0 
);

mat3 sobel_x = mat3( 
     1.0, 2.0, 1.0, 
     0.0, 0.0, 0.0, 
    -1.0, -2.0, -1.0 
);


float near = 0.1;
float far = 1500.0;

//linearize to get right values
float linearize(float depth){
	float z = depth * 2.0 - 1.0;
	return ((2.0 * near * far) / (far + near - z * (far - near)));
}


void main()
{ 
//
	vec2 offset = 1.0 / textureSize(depthTexture, 0);

	//offset array for neighbour pixel
	vec2 offsets[9] = vec2[] (
        vec2(-offset.x, offset.y),	// top-left
        vec2(0.0f, offset.y),		// top-center
        vec2(offset.x, offset.y),	// top-right
        vec2(-offset.x, 0.0f),		// center-left
        vec2( 0.0f,  0.0f),			// center-center
        vec2( offset.x, 0.0f),		// center-right
        vec2(-offset.x, -offset.y), // bottom-left
        vec2( 0.0f, -offset.y),		// bottom-center
        vec2( offset.x, -offset.y)  // bottom-right    
	);

	float I[9];
	for(int i = 0; i < 9; i++){
		I[i] = linearize(texture(depthTexture, TexCoords.xy + offsets[i]).r)/far;

	}


	float gx = 0.0;
	float gy = 0.0;

	for(int i = 0; i < 9; i++){
		 gx += I[i] * sx[i];
		 gy += I[i] * sy[i];
	}

	float g = sqrt(pow(gx, 2.0) + pow(gy, 2.0));
	//g = g > 1.0 ? 1.0 : 0.0;
	//g = smoothstep(0.01, 0.1, g);

	vec3 colorDiff = texture(colorTexture, TexCoords).rgb;
	//FragColor = vec4(colorDiff - vec3(g), 1.0);

	vec3 col = texture(colorTexture, TexCoords).rgb;
    //FragColor = vec4(col, 1.0);
	//FragColor = vec4(0, 1, 0, 1);

	FragColor = vec4(vec3(g), 1.0);
	//FragColor=texture(colorTexture,TexCoords);
	//FragColor=texture(depthTexture,TexCoords);
	//FragColor = mix(texture(colorTexture, TexCoords),vec4(vec3(0.0),1.0),vec4(vec3(g), 1.0));


/*
	vec3 colorDiff = texture(colorTexture, TexCoords).rgb;
    mat3 I;
    vec3 texel;
   for (int i=0; i<3; i++) { 
    for (int j=0; j<3; j++) {
   
		vec3 test = texelFetch(depthTexture, ivec2(gl_FragCoord) + ivec2(i-1, j-1), 0).rgb; 
		vec3 tempvec = vec3(linearize(test.x)); 
		I[i][j] = length(tempvec);

    }
}

    float gx = dot(sobel_x[0], I[0]) + dot(sobel_x[1], I[1]) + dot(sobel_x[2], I[2]); 
    float gy = dot(sobel_y[0], I[0]) + dot(sobel_y[1], I[1]) + dot(sobel_y[2], I[2]);

    float g = sqrt(pow(gx, 2.0)+pow(gy, 2.0));

	//g = g > 1.0 ? 1.0 : 0.0;
	//g = smoothstep(0.01, 0.1, g);


	//FragColor = vec4(vec3(g), 1.0);
	

	//FragColor = mix(texture(colorTexture, TexCoords),vec4(vec3(0.0),1.0),vec4(vec3(g), 1.0));
	FragColor = vec4(colorDiff - vec3(g), 1.0);
*/

}


