#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	//vec2 texCoord_world;
	vec4 FragPosLightSpace;
} vert;

out vec4 color;

uniform vec3 camera_world;

//uniform sampler2D colorTexture;
//uniform sampler2D texture_diffuse0;
//uniform sampler2D texture_specular0;
uniform samplerCube cubemap;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform vec3 diffuseColor;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirLight;

uniform float illuminationMultiplier;

uniform sampler2D shadowMap;


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}


float ShadowCalculation(vec4 fragPosLightSpace)
{
    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    // transform to [0,1] range
    projCoords = projCoords * 0.5 + 0.5;
    // get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
    float closestDepth = texture(shadowMap, projCoords.xy).r; 
    // get depth of current fragment from light's perspective
    float currentDepth = projCoords.z;
    // check whether current frag pos is in shadow
	float bias = max(0.001 * (1.0 - dot(vert.normal_world, dirLight.direction)), 0.0003);  
	//bias = 0.001;
    float shadow = 0.0;
	vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
	for(int x = -1; x <= 1; ++x)
	{
		for(int y = -1; y <= 1; ++y)
		{
		    float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
		    shadow += currentDepth - bias > pcfDepth ? 1.0 : 0.0;        
		}    
	}
	shadow /= 9.0;

    return shadow;
}  

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	//Environmental Mapping
	vec3 I = normalize(vert.position_world - camera_world);
    vec3 R = reflect(I, normalize(vert.normal_world));

	color = vec4(diffuseColor * materialCoefficients.x, 1); // ambient
	
	// add directional light contribution
	float shadow = ShadowCalculation(vert.FragPosLightSpace);
	//shadow = 1.0;
	color.rgb += phong(n, -dirLight.direction, v, dirLight.color * diffuseColor, materialCoefficients.y, dirLight.color, materialCoefficients.z, specularAlpha, false, vec3(0)) * (1.0 - shadow);
	vec4 skyBox = vec4(texture(cubemap, R).rgb, 1.0);
	color = mix(color, skyBox, 0.5);
	color = vec4(color.rgb * illuminationMultiplier, color.a);


	// add point light contribution
	//color.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * diffuseColor, materialCoefficients.y, pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation);
	//color = texture(texture_diffuse0, vert.texCoord_world) * color;
}

